
# q-expansion of newform 320.10.a.c, downloaded from the LMFDB on 05 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 320
weight = 10
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-114], [625], [-4242], [-46208], [115934], [494842], [-1008740], [532554], [-4196390], [3365028], [14931358], [11056262], [-6396794], [35559158], [-39738586], [-85185620], [-45748642], [-45286158], [189967468], [412170946], [-95040840], [261706326], [-19938630], [-19503358], [216671898], [748234114], [1057108842], [2496591070], [1115663866], [-312053842], [-2817011328], [-7153114158], [2523227420], [-1465305050], [-3655152052], [-5551906442], [-16763736634], [-13954932642], [-15700011546], [-25190159660], [-10448152022], [9686250988], [-20443105894], [25243126158], [-8067363400], [-8045889808], [-31861804646], [30262102442], [-20610100590], [3617351226], [-2219162920], [25797693262], [-89184805248], [-110058241158], [-20637427166], [26488963790], [-34400398132], [106022890758], [100298985222], [-2492963874], [84706805694], [6355069042], [-271252675692], [163498922666], [-253606079642], [-240064012728], [-51381151358], [276559585042], [-466592483350], [-100997624814], [166447470120], [297380650758], [-195713508346], [-167009273660], [-72078190326], [-792249454170], [147287630558], [-322101072798], [-396422948770], [-1231616918140], [1175003664518], [232327085628], [-151553685374], [521385143480], [1101694717206], [585671911750], [-570803733158], [760278773058], [714289085634], [782520703442], [695738310360], [1651955550358], [1199886422912], [1431456972700], [-1418329912286], [1164633994970], [-836945778118], [-2602881627554], [-3074063734062], [1329717200842], [-2494179523842], [1066891376766], [-4572783918690], [-2323048560568], [1011436366442], [-1101426177558], [984366159506], [-4513967513600], [-1442119385098], [4121054634758], [-1384697869666], [622713642042], [1894376281460], [1101596159228], [-4852519993938], [1574868802206], [-6930253570042], [8154987967414], [-2925528906620], [1629205839158], [-2875208158254], [-3196196521842], [-547901400474], [590995901512], [-7127384745602], [-909268871230], [-10801666430760], [9776913113758], [-352295410826], [-2721242355980], [5853106110794], [5137227734948], [14571278779958], [10393578513642], [-3916636065790], [6368937400254], [655342967242], [-3837988330442], [2675805974330], [-23258587112208], [-16573982509482], [-13733194988246], [18003590919442], [-21111609462690], [3479882780080], [-11268586464786], [3752268361042], [15701443402780], [13931629789834], [15983174933958], [-31622696231378], [-14852571551074], [9393247844958], [26442903360642], [34164452537108], [-31436645730160], [-8770070949610], [-16629351916758], [7560516404338], [34092472426242], [34588514233786], [-45989111843642], [27266283848232], [-49222819899758], [-39833691832926], [25638770426788], [49724096641158], [5938496734130], [10455969048334], [26253731968260], [-19180586075182], [71844237275028], [43589469701826], [31843307784080], [18413362607850], [66734989735752], [53374366986858], [-70205768108966], [-46388140333810], [-37129809342842], [-23753560711888], [47552997511094], [66753404219042], [-6723849679686], [3022641984770], [32334313058958], [18129148806046], [-59610110499010], [17309743408148], [-73437791070614], [-62263112815034], [76890043425432], [-44422663886722], [11613016012842], [-43458616881094], [-88589515314098], [5194028728134], [-148108156412958], [-87584920280846], [24170800350810], [62015899349628], [-80705190698442], [-164943529871150], [93863528639580], [-121479643282842], [-181947583319840], [-100377919768274], [-46295747749830], [-114003124801288], [5095318031642], [154232972753398], [-82780002159486], [-91599630891158], [96818771717040], [-30201163310218], [-22771201312642], [103376062986642], [149806836257758], [-84442094356146], [-78356084518522], [-65159480103200], [75131107456530], [-268665927007246], [73248132482042], [175506794611510], [17537800481826], [-215509743000320], [-224618347344442], [-120389985541848], [139356256998414], [-159037977177620], [108783155696668], [-203263080866378], [-341231455087474], [137707180404558], [-103265883486330], [-9450597776906], [265155731086500], [-139503900644892], [129766769475846], [206409387319872], [-66359695328806], [-421348419014550], [-155935873517214], [286390190356520], [16143694037758], [57190857594232], [-395212267973860], [-404342324742526], [29889275226158], [-227229646831598], [280798770098358], [329028860425230], [3154273032934], [47491097526060], [-219892256841382], [223469056295042], [-142428792290442], [-483016834548358], [509360986774634], [412635134193242], [356674260902290], [-312317264023506], [324569432380842], [-487437352747700], [213508400718170], [457803894242282], [-529030310202154], [-52510706891026], [351103663183138], [-685656428569358], [145584343853786], [203093037701520], [490498675934442], [-759323548398326], [385510974456642], [-390819216866070], [-619101417585498], [92676516637392], [914398682647354], [-201278427658972], [-787383020891642], [26732999338458], [865906740858042], [202667005014468], [346198926437146], [-1438032917442], [406989570675760], [-397072715863530], [-318171048433202], [407653770167642], [-307598807996334], [538276069069872], [261751142680774], [19161490351950], [-1116330671611052], [1160141273196654], [410714653240740], [-1251785319370558], [671283362424306], [-1033970092945642], [-539657076538600], [1270367591756286], [-889107799469408], [-655865653013758], [-588854685679758], [775101952706410], [1090164660323880], [-329946740793186], [-165096705392166], [-1368787095379710], [421647558788322], [-481682947460674], [639437887564558], [-164729673344630], [-1498420592199300], [277676998981908], [-179350506296934], [-245963526447210], [-1442055328326328], [-43371352512958], [969269553658238], [333738678462194], [-422764872466014], [-122466094757858], [1224450848008940], [1105472926602886], [1791271158353958], [-1626473692167466], [658522023462318], [-87078891997642], [-1467812074637720], [423465236468006], [-1779291235736848], [-1930158119175958], [-506932049619410], [-82892314178854], [338635734020822], [-933848012092242], [1379510611096094], [1728117388049642], [1947292487277270], [-936550553690292], [369861224746574], [-324661395559380], [525758683908838], [-1248159756924158], [63225405028548], [-1169719871078642], [1474244303287632], [-1563313744336758], [-1551048905799122], [-1631002916297326], [1298380031403330], [-2372825797646094], [-1934259895685000], [169579112681592], [-1318194118577358], [2621758296073754], [-1049637744092842], [3060639102423562], [2472936467422758], [-1654385350921220], [439010270456042], [1668162791435146], [1000478284482558], [1484057189044914], [-1542903399076618], [-1629571838501328], [-959120896847580], [2938096459780994], [-3429835825160250], [1696761549099948], [-2520473425177042], [-3630548256776660], [1883114254586388], [3024742623946706], [-3377493202502970], [-1328454668244158], [-2549948551584782], [-2497063832056974], [527187785545958], [2434115970765842], [1326091895687580], [3289144812735434], [-400879007916932], [4884048334437358], [-3299674717955274], [2139094256501958], [4140324165878070], [-87334051456906], [4076784828802300], [157018321518242], [-2412110379636292], [4647404545634066], [4450741161517240], [325504424613690], [1661556810016672], [-1905143433505962], [-3359832760925650], [-1266728416754214], [-5092800477749242], [-240716715434158], [5331637083778630], [-2219643090476212], [2375141483736358], [1568123767904402], [-2994484857298314], [1267192385735460], [-2846821198252974], [-2246253609472242], [-846118855747394], [-611797891306848], [6128291746538642], [-5025547471803742], [-1714579557503840], [-3838050106753042], [-4659063807808158], [-2609310194047286], [452652425427070], [-389248279679842], [5887152731296758], [2419797063254620], [2821844660046986], [3615761714015558], [2879337562336766], [4522011165614910], [397754883289832], [-5993784409052200]]
